@@ -76,12 +76,13 @@ namespace Beagle.Daemon {
 				}
 			}
 
-			foreach (DictionaryEntry entry in matched_filters_by_flavor) {
-				FilterFlavor flav = (FilterFlavor) entry.Key;
-				Filter filter = (Filter) entry.Value;
-				
-				if (Debug)
+			if (Debug) {
+				foreach (DictionaryEntry entry in matched_filters_by_flavor) {
+					FilterFlavor flav = (FilterFlavor) entry.Key;
+					Filter filter = (Filter) entry.Value;
+					
 					Logger.Log.Debug ("Found matching filter: {0}, Weight: {1}", filter, flav.Weight);
+				}
 			}
 
 			return matched_filters_by_flavor.Values;
@@ -248,6 +249,8 @@ namespace Beagle.Daemon {
 					succesful_open = candidate_filter.Open (binary_stream);
 					
 				if (succesful_open) {
+					if (candidate_filter.Timestamp != DateTime.MinValue)
+						indexable.Timestamp = candidate_filter.Timestamp;
 					foreach (Property prop in candidate_filter.Properties)
 						indexable.AddProperty (prop);
 					indexable.SetTextReader (candidate_filter.GetTextReader ());
