@@ -174,8 +174,7 @@ namespace Beagle.Daemon {
 					try {
 						iq = Activator.CreateInstance (type) as IQueryable;
 					} catch (Exception e) {
-						Logger.Log.Error ("Caught exception while instantiating {0} backend", flavor.Name);
-						Logger.Log.Error (e);
+						Logger.Log.Error (e, "Caught exception while instantiating {0} backend", flavor.Name);
 					}
 
 					if (iq != null) {
@@ -306,8 +305,7 @@ namespace Beagle.Daemon {
 				Logger.Log.Warn ("Unable to create read-only index (likely due to index version mismatch): {0}", index_dir.FullName);
 				return false;
 			} catch (Exception e) {
-				Logger.Log.Error ("Caught exception while instantiating static queryable: {0}", index_dir.Name);
-				Logger.Log.Error (e);					
+				Logger.Log.Error (e, "Caught exception while instantiating static queryable: {0}", index_dir.Name);
 				return false;
 			}
 			
@@ -329,12 +327,18 @@ namespace Beagle.Daemon {
 
 		////////////////////////////////////////////////////////
 
-		static public void Start ()
+		private static ArrayList assemblies = null;
+
+		// Perform expensive initialization steps all at once.
+		// Should be done before SignalHandler comes into play.
+		static public void Init ()
 		{
 			ReadBackendsFromConf ();
+			assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
+		}
 
-			ArrayList assemblies = ReflectionFu.ScanEnvironmentForAssemblies ("BEAGLE_BACKEND_PATH", PathFinder.BackendDir);
-
+		static public void Start ()
+		{
 			// Only add the executing assembly if we haven't already loaded it.
 			if (assemblies.IndexOf (Assembly.GetExecutingAssembly ()) == -1)
 				assemblies.Add (Assembly.GetExecutingAssembly ());
@@ -613,20 +617,10 @@ namespace Beagle.Daemon {
 
 		////////////////////////////////////////////////////////
 
-		static public string GetIndexInformation ()
+		static public IEnumerable GetIndexInformation ()
 		{
-			StringBuilder builder = new StringBuilder ('\n');
-
-			foreach (Queryable q in queryables) {
-				QueryableStatus status = q.GetQueryableStatus ();
-
-				builder.Append ("Name: ").Append (status.Name).Append ('\n');
-				builder.Append ("Count: ").Append (status.ItemCount).Append ('\n');
-				builder.Append ("Indexing: ").Append (status.IsIndexing).Append ('\n');
-				builder.Append ('\n');
-			}
-
-			return builder.ToString ();
+			foreach (Queryable q in queryables)
+				yield return q.GetQueryableStatus ();
 		}
 
 		////////////////////////////////////////////////////////
